@@ -1,30 +1,29 @@
-/* This file is part of the SpeedCrunch project
-   Copyright (C) 2004 Ariya Hidayat <ariya@kde.org>
-                 2005-2006 Johan Thelin <e8johan@gmail.com>
-                 2007 Helder Correia <helder.pereira.correia@gmail.com>
+// This file is part of the SpeedCrunch project
+// Copyright (C) 2004 Ariya Hidayat <ariya@kde.org>
+// Copyright (C) 2005-2006 Johan Thelin <e8johan@gmail.com>
+// Copyright (C) 2007-2008 Helder Correia <helder.pereira.correia@gmail.com>
+//
+// This program is free software; you can redistribute it and/or
+// modify it under the terms of the GNU General Public License
+// as published by the Free Software Foundation; either version 2
+// of the License, or (at your option) any later version.
+//
+// This program is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with this program; see the file COPYING.  If not, write to
+// the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+// Boston, MA 02110-1301, USA.
 
-   This program is free software; you can redistribute it and/or
-   modify it under the terms of the GNU General Public License
-   as published by the Free Software Foundation; either version 2
-   of the License, or (at your option) any later version.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
-
-   You should have received a copy of the GNU General Public License
-   along with this program; see the file COPYING.  If not, write to
-   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.
- */
-#include <base/evaluator.hxx>
-#include <base/functions.hxx>
-#include <base/settings.hxx>
-
-#include <float.h>
-#include <limits.h>
-#include <math.h>
+#include "evaluator.hxx"
+#include "functions.hxx"
+#ifdef _BISON
+# include "bison/bisonparser.cpp"
+#endif
 
 #include <QApplication>
 #include <QLocale>
@@ -34,6 +33,7 @@
 #include <QVector>
 #include <QStack>
 
+
 // #define EVALUATOR_DEBUG
 
 #ifdef EVALUATOR_DEBUG
@@ -54,7 +54,8 @@ class Opcode
 {
 public:
 
-  enum { Nop = 0, Load, Ref, Function, Add, Sub, Neg, Mul, Div, Pow, Fact, Modulo, IntDiv };
+  enum { Nop = 0, Load, Ref, Function, Add, Sub, Neg, Mul, Div, Pow, Fact,
+         Modulo, IntDiv };
 
   unsigned type;
   unsigned index;
@@ -64,25 +65,26 @@ public:
   Opcode( unsigned t, unsigned i ): type(t), index(i) {};
 };
 
-class EvaluatorPrivate
-{
-public:
-  Evaluator *evaluator;
-  bool dirty;
-  bool valid;
-  QString expression;
 
+struct Evaluator::Private
+{
+  Evaluator * p;
+  Functions * functions;
+
+  char    radixChar;
+  bool    dirty;
+  bool    valid;
+  QString expression;
   QString error;
-  QMap<QString,Variable> variables;
-  Evaluator::AngleMode angleMode;
-  QString decimalPoint;
 
   QString assignId;
   QVector<Opcode> codes;
   QStringList identifiers;
   QVector<HNumber> constants;
+  QMap<QString,Variable> variables;
 };
 
+
 class TokenStack : public QVector<Token>
 {
 public:
@@ -202,21 +204,24 @@ HNumber Token::asNumber() const
 Token::Op Token::asOperator() const
 {
   if( isOperator() )
-		return matchOperator( m_text );
+    return matchOperator( m_text );
   else
-		return InvalidOp;
+    return InvalidOp;
 }
 
 QString Token::description() const
 {
   QString desc;
 
-  switch (m_type )
+  switch ( m_type )
   {
-    case Number     : desc = "Number"    ; break;
-    case Identifier : desc = "Identifier"; break;
-    case Operator   : desc = "Operator"  ; break;
-    default         : desc = "Unknown"   ; break;
+    case stxNumber     : desc = "Number"    ; break;
+    case stxIdentifier : desc = "Identifier"; break;
+    case stxOpenPar    : //refty
+    case stxClosePar   :
+    case stxSep        :
+    case stxOperator   : desc = "Operator"  ; break;
+    default            : desc = "Unknown"   ; break;
   }
 
   while( desc.length() < 10 )
@@ -284,14 +289,62 @@ static bool isIdentifier( QChar ch )
 
 // Constructor
 
-Evaluator::Evaluator()
+Evaluator::Evaluator( Functions * f, char radixChar, QObject * parent )
+  : QObject( parent ), d( new Evaluator::Private )
 {
-  d = new EvaluatorPrivate;
+  d->p = this;
+  d->functions = f;
+
+  if ( radixChar == 'C' )
+    d->radixChar = QLocale().decimalPoint().toAscii();
+  else
+    d->radixChar = radixChar;
+
   clear();
+
+#ifdef _BISON
+  QStringList script;
+  script << "\\escape \\(\\\"@\\\"\\)"
+         << "@def @(@\"\"@\"@;@\"\"@\"@;@\"\"@\"@)"
+         << "@def @(\"(\"@;\"(\"@;\")\"@)"
+         << "@def (\";\"@;\";\")"
+         << "@def (\"def\";\"def\")"
+
+         << "def(\"escape\";\"esc\")"
+         << "esc \"\\\""
+         << "def(\",\";\"0,\")"
+         << "\\undef (\"esc\"; 0,00)"
+         << "\\def (\".\";\".\")"
+
+         << "\\def (\".\";\"0.\")"
+         << "\\def (\"e\";\"0e\")"
+         << "\\def (\"p\";\"0p\")"
+         << "\\def (\"x\";\"0x\")"
+         << "\\def (\"xs\";\"0xs\")"
+
+         << "\\sin 0.3e1"
+         << "0x2Fp2"
+         << "0xsF0"
+         << "\\def (\"posscale\"; \"1+\")"
+         << "\\def (\"negscale\"; \"1-\")"
+
+         << "2e-1"
+         << "\\def (\"-\";\"-\")"
+         << "-1"
+         << "\\overload (\"add\";\"-\")"
+         << "-1-2"
+
+         << "\\def (\"!\";\"!\")"
+         << "-1!"
+         << "\\def (\"pi\";\"pi\")"
+         << "\\def (\"sin\";\"sin\")"
+         << "sin pi"
+     ;
+  SglExprLex::self().run(script);
+#endif
 }
 
 // Destructor
-
 Evaluator::~Evaluator()
 {
   delete d;
@@ -300,9 +353,11 @@ Evaluator::~Evaluator()
 // Sets a new expression
 // note that both the real lex and parse processes will happen later on
 // when needed (i.e. "lazy parse"), for example during evaluation.
-
 void Evaluator::setExpression( const QString& expr )
 {
+#ifdef _BISON
+  SglExprLex::self().setExpression(expr);
+#endif
   d->expression = expr;
   d->dirty      = true;
   d->valid      = false;
@@ -310,7 +365,6 @@ void Evaluator::setExpression( const QString& expr )
 }
 
 // Returns the expression
-
 QString Evaluator::expression() const
 {
   return d->expression;
@@ -318,12 +372,11 @@ QString Evaluator::expression() const
 
 // Returns the validity
 // note: empty expression is always invalid.
-
 bool Evaluator::isValid() const
 {
   if( d->dirty )
   {
-    Tokens tokens = scan( d->expression, d->decimalPoint );
+    Tokens tokens = scan( d->expression );
     if( !tokens.valid() )
       compile( tokens );
     else
@@ -333,26 +386,19 @@ bool Evaluator::isValid() const
 }
 
 // Clears everything, also mark it as invalid.
-
 void Evaluator::clear()
 {
-  d->expression = QString();
-  d->dirty      = true;
-  d->valid      = false;
-
-  d->error        = QString();
-  d->angleMode    = Degree;
-  d->decimalPoint = QString();
-
+  d->expression  = QString();
+  d->dirty       = true;
+  d->valid       = false;
+  d->error       = QString();
   d->constants.clear();
   d->codes.clear();
-  d->assignId = QString();
-
+  d->assignId    = QString();
   clearVariables();
 }
 
 // Returns error message
-
 QString Evaluator::error() const
 {
   return d->error;
@@ -362,30 +408,21 @@ QString Evaluator::error() const
 // this triggers again the lexical analysis step. it is however preferable
 // (even when there's small performance penalty) because otherwise we need to
 // store parsed tokens all the time which serves no good purpose.
-
 Tokens Evaluator::tokens() const
 {
-  return scan( d->expression, d->decimalPoint );
+  return scan( d->expression );
 }
 
-Tokens Evaluator::scan( const QString& expr, const QString& settingsDecimal )
+Tokens Evaluator::scan( const QString& expr ) const
 {
   // to hold the result
   Tokens tokens;
 
   // auto-detect, always dot, or always comma
-
-  QChar decimalPoint;
   QChar wrongDecimalPoint;
-
-  if( settingsDecimal.length() == 1 )
-    decimalPoint = settingsDecimal[0];
-  else
-    decimalPoint = QLocale().decimalPoint();
-
+  //decimalPoint = Settings::self()->dot();
   // sanity check for wrong decimal separator usage
-
-  if ( decimalPoint == ',' )
+  if ( d->radixChar == ',' )
     wrongDecimalPoint = '.';
   else
     wrongDecimalPoint = ',';
@@ -405,6 +442,7 @@ Tokens Evaluator::scan( const QString& expr, const QString& settingsDecimal )
   QString ex = expr;
   QString tokenText;
   int tokenStart = 0;
+  Token::Type type;
 
   // force a terminator
   ex.append( QChar() );
@@ -437,7 +475,7 @@ Tokens Evaluator::scan( const QString& expr, const QString& settingsDecimal )
        }
 
        // decimal dot ?
-       else if ( ch == decimalPoint )
+       else if ( ch == d->radixChar )
        {
          tokenText.append( ex.at(i++) );
          state = InDecimal;
@@ -477,9 +515,16 @@ Tokens Evaluator::scan( const QString& expr, const QString& settingsDecimal )
          // any matched operator ?
          if( op != Token::InvalidOp )
          {
+           switch(op) //refty
+           {
+             case Token::LeftPar:   type = Token::stxOpenPar; break;
+             case Token::RightPar:  type = Token::stxClosePar; break;
+             case Token::Semicolon: type = Token::stxSep; break;
+             default: type = Token::stxOperator;
+           }
            int len = s.length();
            i += len;
-           tokens.append( Token( Token::Operator, s.left( len ), tokenStart ) );
+           tokens.append( Token( type, s.left( len ), tokenStart ) );
          }
          else state = Bad;
         }
@@ -501,7 +546,7 @@ Tokens Evaluator::scan( const QString& expr, const QString& settingsDecimal )
        // we're done with identifier
        else
        {
-         tokens.append( Token( Token::Identifier, tokenText, tokenStart ) );
+         tokens.append( Token( Token::stxIdentifier, tokenText, tokenStart ) );
          tokenStart = i;
          tokenText = "";
          state = Start;
@@ -514,8 +559,8 @@ Tokens Evaluator::scan( const QString& expr, const QString& settingsDecimal )
        // consume as long as it's digit
        if( ch.isDigit() ) tokenText.append( ex.at(i++) );
 
-	   // convert decimal separator to '.'
-       else if( ch == decimalPoint )
+       // convert decimal separator to '.'
+       else if( ch == d->radixChar )
        {
          tokenText.append( '.' );
          i++;
@@ -550,7 +595,7 @@ Tokens Evaluator::scan( const QString& expr, const QString& settingsDecimal )
        // we're done with integer number
        else
        {
-         tokens.append( Token( Token::Number, tokenText, tokenStart ) );
+         tokens.append( Token( Token::stxNumber, tokenText, tokenStart ) );
          tokenText = ""; state = Start;
        }
        break;
@@ -560,7 +605,7 @@ Tokens Evaluator::scan( const QString& expr, const QString& settingsDecimal )
           tokenText.append( ex.at(i++).toUpper() );
        else // we're done with hexa number
        {
-         tokens.append( Token( Token::Number, tokenText, tokenStart ) );
+         tokens.append( Token( Token::stxNumber, tokenText, tokenStart ) );
          tokenText = ""; state = Start;
        }
        break;
@@ -569,7 +614,7 @@ Tokens Evaluator::scan( const QString& expr, const QString& settingsDecimal )
           tokenText.append( ex.at(i++) );
        else // we're done with binary number
        {
-         tokens.append( Token( Token::Number, tokenText, tokenStart ) );
+         tokens.append( Token( Token::stxNumber, tokenText, tokenStart ) );
          tokenText = "";
          state = Start;
        }
@@ -579,7 +624,7 @@ Tokens Evaluator::scan( const QString& expr, const QString& settingsDecimal )
           tokenText.append( ex.at(i++) );
        else // we're done with octal number
        {
-         tokens.append( Token( Token::Number, tokenText, tokenStart ) );
+         tokens.append( Token( Token::stxNumber, tokenText, tokenStart ) );
          tokenText = ""; state = Start;
        }
        break;
@@ -600,7 +645,7 @@ Tokens Evaluator::scan( const QString& expr, const QString& settingsDecimal )
        // we're done with floating-point number
        else
        {
-         tokens.append( Token( Token::Number, tokenText, tokenStart ) );
+         tokens.append( Token( Token::stxNumber, tokenText, tokenStart ) );
          tokenText = "";
          state = Start;
        };
@@ -627,7 +672,7 @@ Tokens Evaluator::scan( const QString& expr, const QString& settingsDecimal )
        // we're done with floating-point number
        else
        {
-         tokens.append( Token( Token::Number, tokenText, tokenStart ) );
+         tokens.append( Token( Token::stxNumber, tokenText, tokenStart ) );
          tokenText = "";
          state = Start;
        };
@@ -672,11 +717,10 @@ void Evaluator::compile( const Tokens& tokens ) const
   for( int i = 0; i <= tokens.count(); i++ )
   {
     // helper token: InvalidOp is end-of-expression
-    Token token =  ( i < tokens.count() ) ? tokens[i] : Token( Token::Operator );
+    Token token =  ( i < tokens.count() ) ? tokens[i] : Token( Token::stxOperator );
     Token::Type tokenType = token.type();
-
-#ifdef EVALUATOR_DEBUG
-#endif
+    if (tokenType >= Token::stxOperator)
+      tokenType = Token::stxOperator;
 
 #ifdef EVALUATOR_DEBUG
     dbg << "\n";
@@ -684,11 +728,11 @@ void Evaluator::compile( const Tokens& tokens ) const
 #endif
 
     // unknown token is invalid
-    if( tokenType == Token::Unknown ) break;
+    if( tokenType == Token::stxUnknown ) break;
 
     // for constants, push immediately to stack
     // generate code to load from a constant
-    if ( tokenType == Token::Number )
+    if ( tokenType == Token::stxNumber )
     {
       syntaxStack.push( token );
       d->constants.append( token.asNumber() );
@@ -700,7 +744,7 @@ void Evaluator::compile( const Tokens& tokens ) const
 
     // for identifier, push immediately to stack
     // generate code to load from reference
-    if( tokenType == Token::Identifier )
+    if( tokenType == Token::stxIdentifier )
     {
       syntaxStack.push( token );
       d->identifiers.append( token.text() );
@@ -711,7 +755,7 @@ void Evaluator::compile( const Tokens& tokens ) const
     }
 
     // special case for percentage
-    if( tokenType == Token::Operator )
+    if( tokenType == Token::stxOperator )
     if( token.asOperator() == Token::Percent )
     if( syntaxStack.itemCount() >= 1 )
     if( !syntaxStack.top().isOperator() )
@@ -725,7 +769,7 @@ void Evaluator::compile( const Tokens& tokens ) const
     }
 
     // for any other operator, try to apply all parsing rules
-    if( tokenType == Token::Operator )
+    if( tokenType == Token::stxOperator )
     if( token.asOperator() != Token::Percent )
     {
 #ifdef EVALUATOR_DEBUG
@@ -769,7 +813,7 @@ void Evaluator::compile( const Tokens& tokens ) const
         // if token is operator, and stack already has: id ( arg
         if( !ruleFound )
         if( !argHandled )
-        if( tokenType == Token::Operator )
+        if( tokenType == Token::stxOperator )
         if( syntaxStack.itemCount() >= 3 )
         {
             Token arg = syntaxStack.top();
@@ -789,6 +833,53 @@ void Evaluator::compile( const Tokens& tokens ) const
             }
         }
 
+        // rule for postfix operators
+            if( !ruleFound )
+            if( syntaxStack.itemCount() >= 2 )
+            {
+              Token postfix = syntaxStack.top();
+              Token y = syntaxStack.top( 1 );
+              if ( postfix.isOperator() )
+              if ( !y.isOperator() )
+              switch ( postfix.asOperator() )
+              {
+                case Token::Exclamation:
+                  ruleFound = true;
+                  syntaxStack.pop();
+                  syntaxStack.pop();
+                  syntaxStack.push(y);
+                  d->codes.append( Opcode( Opcode::Fact ) );
+#ifdef EVALUATOR_DEBUG
+                  dbg << "    Rule for postfix operator" << "\n";
+#endif
+                  break;
+                  default:;
+              }
+        }
+
+        // rule for parenthesis:  ( Y ) -> Y
+            if( !ruleFound )
+            if( syntaxStack.itemCount() >= 3 )
+            {
+              Token right = syntaxStack.top();
+              Token y = syntaxStack.top( 1 );
+              Token left = syntaxStack.top( 2 );
+              if( right.isOperator() )
+              if( !y.isOperator() )
+              if( left.isOperator() )
+              if( right.asOperator() == Token::RightPar )
+              if( left.asOperator() == Token::LeftPar )
+              {
+                ruleFound = true;
+                syntaxStack.pop();
+                syntaxStack.pop();
+                syntaxStack.pop();
+                syntaxStack.push( y );
+#ifdef EVALUATOR_DEBUG
+                dbg << "    Rule for (Y) -> Y" << "\n";
+#endif
+              }
+            }
 
         // rule for simplified syntax for function e.g. "sin pi" or "cos 1.2"
         // i.e no need for parentheses like "sin(pi)" or "cos(1.2)"
@@ -799,7 +890,7 @@ void Evaluator::compile( const Tokens& tokens ) const
           Token id = syntaxStack.top( 1 );
           if( !arg.isOperator() )
           if( id.isIdentifier() )
-          if( FunctionRepository::self()->function( id.text() ) )
+          if( d->functions->function( id.text() ) )
           {
             ruleFound = true;
             d->codes.append( Opcode( Opcode::Function, 1 ) );
@@ -823,7 +914,7 @@ void Evaluator::compile( const Tokens& tokens ) const
             if( !x.isOperator() )
             if( op.isOperator() )
             if( id.isIdentifier() )
-            if( FunctionRepository::self()->function( id.text() ) )
+            if( d->functions->function( id.text() ) )
             if( ( op.asOperator() == Token::Plus ) ||
                ( op.asOperator() == Token::Minus ) )
             {
@@ -839,6 +930,7 @@ void Evaluator::compile( const Tokens& tokens ) const
             }
           }
 
+#if 0
          // rule for unary postfix operator in simplified function syntax
          // this handles case like "sin 90!"
          if( !ruleFound )
@@ -847,7 +939,7 @@ void Evaluator::compile( const Tokens& tokens ) const
             Token op = syntaxStack.top();
             Token x = syntaxStack.top( 1 );
             Token id = syntaxStack.top( 2 );
-            if( id.isIdentifier() && FunctionRepository::self()->function( id.text() ) )
+            if( id.isIdentifier() && d->functions->function( id.text() ) )
             {
                if( !x.isOperator() && op.isOperator() &&
                   op.asOperator() == Token::Exclamation )
@@ -863,14 +955,14 @@ void Evaluator::compile( const Tokens& tokens ) const
                }
             }
          }
-
+#endif
 
         // rule for function arguments, if token is , or )
         // id ( arg1 ; arg2 -> id ( arg
         if( !ruleFound )
         if( syntaxStack.itemCount() >= 5 )
         if( ( token.asOperator() == Token::RightPar ) ||
-        ( token.asOperator() == Token::Semicolon ) )
+            ( token.asOperator() == Token::Semicolon ) )
         {
           Token arg2 = syntaxStack.top();
           Token sep = syntaxStack.top( 1 );
@@ -910,7 +1002,7 @@ void Evaluator::compile( const Tokens& tokens ) const
             syntaxStack.pop();
             syntaxStack.pop();
             syntaxStack.pop();
-            syntaxStack.push( Token( Token::Number ) );
+            syntaxStack.push( Token( Token::stxNumber ) );
             d->codes.append( Opcode( Opcode::Function, 0 ) );
 #ifdef EVALUATOR_DEBUG
           dbg << "    Rule for function call with parentheses, but without argument" << "\n";
@@ -918,30 +1010,6 @@ void Evaluator::compile( const Tokens& tokens ) const
           }
         }
 
-        // rule for parenthesis:  ( Y ) -> Y
-        if( !ruleFound )
-        if( syntaxStack.itemCount() >= 3 )
-        {
-          Token right = syntaxStack.top();
-          Token y = syntaxStack.top( 1 );
-          Token left = syntaxStack.top( 2 );
-          if( right.isOperator() )
-          if( !y.isOperator() )
-          if( left.isOperator() )
-          if( right.asOperator() == Token::RightPar )
-          if( left.asOperator() == Token::LeftPar )
-          {
-            ruleFound = true;
-            syntaxStack.pop();
-            syntaxStack.pop();
-            syntaxStack.pop();
-            syntaxStack.push( y );
-#ifdef EVALUATOR_DEBUG
-          dbg << "    Rule for (Y) -> Y" << "\n";
-#endif
-          }
-        }
-
         // rule for binary operator:  A (op) B -> A
         // conditions: precedence of op >= precedence of token
         // action: push (op) to result
@@ -1002,6 +1070,7 @@ void Evaluator::compile( const Tokens& tokens ) const
                if( op2.asOperator() == Token::Minus )
                   d->codes.append( Opcode( Opcode::Neg ) );
             }
+#if 0
             else // maybe postfix
             {
                x = op2; op2 = syntaxStack.top();
@@ -1012,6 +1081,7 @@ void Evaluator::compile( const Tokens& tokens ) const
                   d->codes.append( Opcode( Opcode::Fact ) );
                }
             }
+#endif
             if (ruleFound)
             {
                syntaxStack.pop();
@@ -1028,7 +1098,7 @@ void Evaluator::compile( const Tokens& tokens ) const
          // action: push (op) to result
          if( !ruleFound )
          if( token.asOperator() != Token::LeftPar )
-         if( token.asOperator() != Token::Caret )
+//         if( token.asOperator() != Token::Caret )
          if( token.asOperator() != Token::Exclamation)
          if( syntaxStack.itemCount() == 2 )
          {
@@ -1062,8 +1132,8 @@ void Evaluator::compile( const Tokens& tokens ) const
             }
           }
 
-        if( !ruleFound ) break;
-      }
+         if( !ruleFound ) break;
+       }
 
       // can't apply rules anymore, push the token
       if( token.asOperator() != Token::Percent )
@@ -1081,6 +1151,7 @@ void Evaluator::compile( const Tokens& tokens ) const
 
 #ifdef EVALUATOR_DEBUG
     dbg << "Dump: " << dump() << "\n";
+    debugFile.close();
 #endif
 
   // bad parsing ? clean-up everything
@@ -1091,32 +1162,9 @@ void Evaluator::compile( const Tokens& tokens ) const
     d->identifiers.clear();
   }
 
-#ifdef EVALUATOR_DEBUG
-  debugFile.close();
-#endif
 }
 
-Evaluator::AngleMode Evaluator::angleMode() const
-{
-  return d->angleMode;
-}
-
-void Evaluator::setAngleMode( AngleMode am )
-{
-  d->angleMode = am;
-}
-
-QString Evaluator::decimalPoint() const
-{
-  return d->decimalPoint;
-}
-
-void Evaluator::setDecimalPoint( const QString& dp )
-{
-  d->decimalPoint = dp;
-}
-
-HNumber Evaluator::eval()
+HNumber Evaluator::evalNoAssign()
 {
   QStack<HNumber> stack;
   QStack<QString> refs;
@@ -1128,12 +1176,12 @@ HNumber Evaluator::eval()
 
   if( d->dirty )
   {
-    Tokens tokens = scan( d->expression, d->decimalPoint );
+    Tokens tokens = scan( d->expression );
 
     // invalid expression ?
     if( !tokens.valid() )
     {
-      d->error = qApp->translate( "evaluator", "invalid expression" );
+      d->error = tr( "invalid expression" );
       return HNumber( 0 );
     }
 
@@ -1151,40 +1199,6 @@ HNumber Evaluator::eval()
     compile( tokens );
   }
 
-  // can not overwrite pi
-  if( d->assignId == QString("pi") )
-  {
-     d->error = d->assignId + ": " + qApp->translate( "evaluator",
-       "variable cannot be overwritten" );
-     return HNumber( 0 );
-  }
-  // can not overwrite phi
-  if( d->assignId == QString("phi") )
-  {
-     d->error = d->assignId + ": " + qApp->translate( "evaluator",
-       "variable cannot be overwritten" );
-     return HNumber( 0 );
-  }
-  // can not overwrite ans
-  if( d->assignId == QString("ans") )
-  {
-     d->error = d->assignId + ": " + qApp->translate( "evaluator",
-       "variable cannot be overwritten" );
-     return HNumber( 0 );
-  }
-
-  // variable can't have the same name as function
-  if( FunctionRepository::self()->function( d->assignId ) )
-  {
-     d->error = d->assignId + ": " +  qApp->translate( "evaluator",
-       "identifier matches an existing function name" );
-     return HNumber( 0 );
-  }
-
-  // magic: always set here to avoid being overwritten by user
-  set( QString("pi"),  HMath::pi() );
-  set( QString("phi"), HMath::phi() );
-
   for( int pc = 0; pc < d->codes.count(); pc++ )
   {
     Opcode& opcode = d->codes[pc];
@@ -1205,11 +1219,11 @@ HNumber Evaluator::eval()
       case Opcode::Neg:
         if( stack.count() < 1 )
         {
-          d->error = qApp->translate( "evaluator", "invalid expression" );
+          d->error = tr( "invalid expression" );
           return HNumber( 0 );
         }
         val1 = stack.pop();
-        val1 = HMath::negate( val1 );
+        val1 = -val1;
         stack.push( val1 );
         break;
 
@@ -1218,7 +1232,7 @@ HNumber Evaluator::eval()
       case Opcode::Add:
         if( stack.count() < 2 )
         {
-          d->error = qApp->translate( "evaluator", "invalid expression" );
+          d->error = tr( "invalid expression" );
           return HNumber( 0 );
         }
         val1 = stack.pop();
@@ -1230,7 +1244,7 @@ HNumber Evaluator::eval()
       case Opcode::Sub:
         if( stack.count() < 2 )
         {
-          d->error = qApp->translate( "evaluator", "invalid expression" );
+          d->error = tr( "invalid expression" );
           return HNumber( 0 );
         }
         val1 = stack.pop();
@@ -1242,7 +1256,7 @@ HNumber Evaluator::eval()
       case Opcode::Mul:
         if( stack.count() < 2 )
         {
-          d->error = qApp->translate( "evaluator", "invalid expression" );
+          d->error = tr( "invalid expression" );
           return HNumber( 0 );
         }
         val1 = stack.pop();
@@ -1254,14 +1268,14 @@ HNumber Evaluator::eval()
       case Opcode::Div:
         if( stack.count() < 2 )
         {
-          d->error = qApp->translate( "evaluator", "invalid expression" );
+          d->error = tr( "invalid expression" );
           return HNumber( 0 );
         }
         val1 = stack.pop();
         val2 = stack.pop();
         if( val1.isZero() )
         {
-          d->error = qApp->translate( "evaluator", "division by zero" );
+          d->error = tr( "division by zero" );
           return HNumber( 0 );
         }
         val2 /= val1;
@@ -1271,7 +1285,7 @@ HNumber Evaluator::eval()
       case Opcode::Pow:
         if( stack.count() < 2 )
         {
-          d->error = qApp->translate( "evaluator", "invalid expression" );
+          d->error = tr( "invalid expression" );
           return HNumber( 0 );
         }
         val1 = stack.pop();
@@ -1283,7 +1297,7 @@ HNumber Evaluator::eval()
       case Opcode::Fact:
         if( stack.count() < 1 )
         {
-          d->error = qApp->translate( "evaluator", "invalid expression" );
+          d->error = tr( "invalid expression" );
           return HNumber( 0 );
         }
         val1 = stack.pop();
@@ -1294,14 +1308,14 @@ HNumber Evaluator::eval()
       case Opcode::Modulo:
         if( stack.count() < 2 )
         {
-          d->error = qApp->translate( "evaluator", "invalid expression" );
+          d->error = tr( "invalid expression" );
           return HNumber( 0 );
         }
         val1 = stack.pop();
         val2 = stack.pop();
         if( val1.isZero() )
         {
-          d->error = qApp->translate( "evaluator", "division by zero" );
+          d->error = tr( "division by zero" );
           return HNumber( 0 );
         }
         val2 = val2 % val1;
@@ -1311,14 +1325,14 @@ HNumber Evaluator::eval()
       case Opcode::IntDiv:
         if( stack.count() < 2 )
         {
-          d->error = qApp->translate( "evaluator", "invalid expression" );
+          d->error = tr( "invalid expression" );
           return HNumber( 0 );
         }
         val1 = stack.pop();
         val2 = stack.pop();
         if( val1.isZero() )
         {
-          d->error = qApp->translate( "evaluator", "division by zero" );
+          d->error = tr( "division by zero" );
           return HNumber( 0 );
         }
         val2 /= val1;
@@ -1336,13 +1350,12 @@ HNumber Evaluator::eval()
         else
         {
           // function
-          function = FunctionRepository::self()->function( fname );
+          function = d->functions->function( fname );
           if( function )
             refs.push( fname );
           else
           {
-            d->error = fname + ": " + qApp->translate( "evaluator",
-              "unknown function or variable" );
+            d->error = fname + ": " + tr( "unknown function or variable" );
             return HNumber( 0 );
           }
         }
@@ -1355,17 +1368,16 @@ HNumber Evaluator::eval()
           break;
 
         fname = refs.pop();
-        function = FunctionRepository::self()->function( fname );
+        function = d->functions->function( fname );
         if( !function )
         {
-          d->error = fname + ": " + qApp->translate( "evaluator",
-            "unknown function or variable" );
+          d->error = fname + ": " + tr( "unknown function or variable" );
           return HNumber( 0 );
         }
 
         if( stack.count() < index )
         {
-          d->error = qApp->translate( "evaluator", "invalid expression" );
+          d->error = tr( "invalid expression" );
           return HNumber( 0 );
         }
 
@@ -1373,7 +1385,7 @@ HNumber Evaluator::eval()
         for( ; index; index-- )
           args.insert( args.begin(), stack.pop() );
 
-        stack.push( function->exec( this, args ) );
+        stack.push( function->exec( args ) );
         if( !function->error().isEmpty() )
         {
           d->error = function->error();
@@ -1390,18 +1402,50 @@ HNumber Evaluator::eval()
   // more than one value in stack ? unsuccesfull execution...
   if( stack.count() != 1 )
   {
-    d->error = qApp->translate( "evaluator", "invalid expression" );
+    d->error = tr( "invalid expression" );
     return HNumber( 0 );
   }
 
   HNumber result = stack.pop();
 
+  return result;
+}
+
+HNumber Evaluator::eval()
+{
+  HNumber result = evalNoAssign(); // this sets d->assignId
+
+  // can not overwrite built-in variables
+  if (    d->assignId == QString("pi")
+       || d->assignId == QString("phi")
+       || d->assignId == QString("ans") )
+  {
+    d->error = d->assignId + ": " + tr( "variable cannot be overwritten" );
+    return HNumber::nan();
+  }
+
+  // variable can't have the same name as function
+  if ( d->functions->function( d->assignId ) )
+  {
+    d->error = d->assignId + ": "
+                 + tr( "identifier matches an existing function name" );
+    return HNumber::nan();
+  }
+
   // handle variable assignment, e.g. "x=2*4"
-  if( !d->assignId.isEmpty() )
+  if ( ! d->assignId.isEmpty() )
     set( d->assignId, result );
 
+  return result;
+}
+
+HNumber Evaluator::evalUpdateAns()
+{
+  HNumber result = eval();
+
   // "ans" is default variable to hold calculation result
-  set( QString("ans"), result );
+  if ( d->error.isEmpty() )
+    set( QString("ans"), result );
 
   return result;
 }
@@ -1442,7 +1486,9 @@ QVector<Variable> Evaluator::variables() const
   for ( it = d->variables.begin(); it != d->variables.end(); ++it )
   {
     Variable var;
+    const char* ASCII;
     var.name = it.value().name;
+    ASCII = var.name.toLatin1();
     var.value = it.value().value;
     result.append( var );
   }
@@ -1452,14 +1498,21 @@ QVector<Variable> Evaluator::variables() const
 
 void Evaluator::clearVariables()
 {
+  HNumber ansBackup = get( QString("ans") );
+
   d->variables.clear();
+
+  set( QString("ans"), ansBackup    );
   set( QString("pi"),  HMath::pi()  );
   set( QString("phi"), HMath::phi() );
-  set( QString("ans"), HNumber(0)   );
 }
 
-QString Evaluator::autoFix( const QString& expr, const QString& decimalPoint )
+QString Evaluator::autoFix( const QString& expr )
 {
+#ifdef _BISON
+//  SglExprLex::self().autoFix(expr);
+#endif
+
   int par = 0;
   QString result;
 
@@ -1476,7 +1529,7 @@ QString Evaluator::autoFix( const QString& expr, const QString& decimalPoint )
     result = result.left( result.length()-1 );
 
   // automagically close all parenthesis
-  Tokens tokens = Evaluator::scan( result, decimalPoint );
+  Tokens tokens = Evaluator::scan( result );
   if(tokens.count())
   {
     for( int i=0; i<tokens.count(); i++ )
@@ -1497,12 +1550,12 @@ QString Evaluator::autoFix( const QString& expr, const QString& decimalPoint )
   // e.g. "cos" is regarded as "cos(ans)"
   if( !result.isEmpty() )
   {
-    Tokens tokens = Evaluator::scan( result, decimalPoint );
+    Tokens tokens = Evaluator::scan( result );
     if( tokens.count() == 1 )
     {
       if( tokens[0].isIdentifier() )
       {
-        Function* f = FunctionRepository::self()->function( tokens[0].text() );
+        Function* f = d->functions->function( tokens[0].text() );
         if( f ) result.append( "(ans)" );
       }
     }
@@ -1511,8 +1564,25 @@ QString Evaluator::autoFix( const QString& expr, const QString& decimalPoint )
   return result;
 }
 
-// Debugging aid
 
+char Evaluator::radixChar() const
+{
+  return d->radixChar;
+}
+
+
+void Evaluator::setRadixChar( char c )
+{
+  if ( c == 'C' )
+    c = QLocale().decimalPoint().toAscii();
+  if ( d->radixChar != c )
+  {
+    d->radixChar = c;
+  }
+}
+
+
+// Debugging aid
 QString Evaluator::dump() const
 {
   QString result;
@@ -1520,7 +1590,7 @@ QString Evaluator::dump() const
 
   if( d->dirty )
   {
-    Tokens tokens = scan( d->expression, d->decimalPoint );
+    Tokens tokens = scan( d->expression );
     compile( tokens );
   }
 
