@@ -1,6 +1,6 @@
 /* floatgamma.h: Gamma function, based on floatnum. */
 /*
-    Copyright (C) 2007 Wolf Lammen.
+    Copyright (C) 2007, 2008 Wolf Lammen.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -42,7 +42,7 @@ char _lngamma(floatnum x, int digits);
 char _gamma(floatnum x, int digits);
 char _gammaint(floatnum integer, int digits);
 char _gamma0_5(floatnum x, int digits);
-char _pochhammer(floatnum x, floatnum n, int digits);
+char _pochhammer(floatnum x, cfloatnum n, int digits);
 
 #ifdef __cplusplus
 }
