@@ -1,6 +1,6 @@
 /* floatio.c: low level conversion, based on floatnum. */
 /*
-    Copyright (C) 2007 Wolf Lammen.
+    Copyright (C) 2007, 2008 Wolf Lammen.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -31,9 +31,10 @@
 
 #include "floatio.h"
 #include "floatlong.h"
+#include <base/errors.h>
 #include <string.h>
 
-#define NORMALBASE (-1)
+typedef enum { NmbNormal, NmbSpecial, NmbBufferOverflow } NmbType;
 
 /* Besides the regular bases 2, 8, 10 and 16, there are
    three others (IO_BASE_DEFAULT, IO_BASE_NAN and IO_BASE_ZERO)
@@ -41,7 +42,7 @@
    pseudo-bases, because you cannot use them as parameter to
    floatnum functions */
 static char
-_ispseudobase(
+_isspecial(
   signed char base)
 {
   switch (base)
@@ -73,7 +74,7 @@ _isempty(
 static char
 _setstr(
   p_buffer dest,
-  char*src)
+  const char* src)
 {
   if (dest->sz > 0)
   {
@@ -191,7 +192,7 @@ _getseqdigit(
 {
   if (ofs < 0 || ofs > _ofslastnz(n))
     return 0;
-  return _ascii2digit(*(char*)(n->param + ofs));
+  return _ascii2digit(*((char*)(n->param) + ofs));
 }
 
 /* returns a digit from an ASCIIZ string containing
@@ -211,26 +212,6 @@ _getcmpldigit(
   return c == n->base? 1 : c;
 }
 
-/* gets the digits of a value encoded as integer */
-static char
-_getexpdigit(
-  int ofs,
-  p_seq_desc n)
-{
-  int count, v;
-  unsigned value, base;
-
-  base = n->base;
-  count = _intdigits(n);
-  v = *(int*)(n->param);
-  value = v < 0? (unsigned)-v : (unsigned)v;
-  if (ofs < 0 || ofs >= count)
-    return 0;
-  for (count = count - ofs; --count > 0;)
-    value /= base;
-  return value % base;
-}
-
 /* initializes a structure used for copying digit
    sequences */
 static void
@@ -247,20 +228,6 @@ _clearint(
 
 /*----------------------   t_[io]token   ------------------*/
 
-/* initializes a t_otokens */
-static void
-_cleartokens(
-  p_otokens tokens)
-{
-  tokens->sign = IO_SIGN_NONE;
-  tokens->base = IO_BASE_NAN;
-  tokens->intpart.sz = 0;
-  tokens->fracpart.sz = 0;
-  tokens->expsign = IO_SIGN_NONE;
-  tokens->expbase = IO_BASE_NAN;
-  tokens->exp.sz = 0;
-}
-
 /* initializes a structure used for describing a
    floating point number */
 void
@@ -271,59 +238,9 @@ _clearnumber(
   n->prefix.base = IO_BASE_NAN;
   _clearint(&n->intpart);
   _clearint(&n->fracpart);
-  n->expbase = IO_BASE_NAN;
   n->exp = 0;
 }
 
-/* creates a sequence descripter from the value of an integer.
-   trailing zeros are not set, n has to be initialized. */
-
-static void
-_value2seq(
-  p_ext_seq_desc n,
-  int* value,
-  int base)
-{
-  /* pre: n is initialized to describe zero,
-          base >= 2
-     post: trailing0 is always 0, regardless of
-           the trailing zeros in value.
-           That's why you cannot use the result in complement mode */
-  int v;
-
-  v = *value;
-  if (v != 0)
-    n->seq.base = base;
-  while (v != 0)
-  {
-    ++n->seq.digits;
-    v /= base;
-  }
-  n->getdigit = _getexpdigit;
-  n->seq.param = value;
-}
-
-/* converts a digit sequence to an integer. Returns
-   0 should the integer overflows */
-static char
-_seq2value(
-  int* value,
-  p_ext_seq_desc n,
-  int sign)
-{
-  int i, ubound;
-
-  *value = 0;
-  ubound = _intdigits(&n->seq);
-  for (i = 0; i < ubound; ++i)
-  {
-    if (!_checkmul(value, n->seq.base)
-        || !_checkadd(value, sign * n->getdigit(i, &n->seq)))
-      return 0;
-  }
-  return 1;
-}
-
 /* creates a sequence descriptor from an ASCII digit
    sequence, not necessarily terminated by \0.
    After maxdigits digits, all following digits are
@@ -370,7 +287,7 @@ _str2seq(
 
 /* copy count digits (leading zeros included) from the
    digit sequence described by n to an ASCII buffer */
-static char
+static Error
 _seq2str(
   p_buffer dest,
   int count,
@@ -381,18 +298,18 @@ _seq2str(
 
   ofs = 0;
   if (count >= dest->sz)
-    return 0;
+    return IOBufferOverflow;
   buf = dest->buf;
   for (; ofs < count; ++ofs)
     *(buf++) = _digit2ascii(n->getdigit(ofs, &n->seq));
   *(buf) = '\0';
-  return 1;
+  return Success;
 }
 
 /* copy all digits (leading zeros included) from the
    digit sequence described by n to an ASCII buffer, but
    complement the sequence before writing to the buffer */
-static char
+static Error
 _cmplseq2str(
   p_buffer dest,
   p_ext_seq_desc n)
@@ -406,7 +323,7 @@ _cmplseq2str(
   buf = dest->buf;
   bound = _intdigits(&n->seq);
   if (bound + 1 > dest->sz)
-    return 0;
+    return IOBufferOverflow;
   lastnz = _ofslastnz(&n->seq);
   for (ofs = -1; ++ofs < lastnz;)
   {
@@ -424,12 +341,12 @@ _cmplseq2str(
   for (; ++ofs < bound;)
     *(buf++) = _digit2ascii(0);
   *(buf) = '\0';
-  return 1;
+  return Success;
 }
 
 /* create a descriptor from a sequence of digits,
    assuming the sequence is an integer */
-static int
+static Error
 str2int(
   p_ext_seq_desc n,
   const char* value,
@@ -439,7 +356,7 @@ str2int(
   char complement;
 
   if (prefix->base == IO_BASE_NAN)
-    return IO_ERROR_NO_BASE;
+    return IONoBase;
   complement = prefix->sign == IO_SIGN_COMPLEMENT;
   if (complement)
   {
@@ -449,46 +366,53 @@ str2int(
   }
   if (value)
     _str2seq(n, value, maxdigits, prefix->base, complement);
-  return IO_NO_ERROR;
+  return Success;
 }
 
 /* if base describes a special value (0 or NaN), the normal
    conversion routines fail. This routine creates special output
    values for these bases, and return
-   0: if the buffer is too small
-   1: if it created output
-   NORMALBASE: if base stands for a usual number that the
+   nmbBufferOverflow: if the buffer is too small
+   nmbSpecial: if it created output
+   nmbNormal: if base stands for a usual number that the
                normal routines should deal with */
-static signed char
-_pseudo2str(
+static NmbType
+_special2str(
   p_otokens tokens,
   signed char base)
 {
+  const char* p;
   switch (base)
   {
   case IO_BASE_ZERO:
-    return _setstr(&tokens->intpart, "0");
+    p = "0";
+    break;
   case IO_BASE_NAN:
-    return _setstr(&tokens->intpart, "NaN");
+    p = "NaN";
+    break;
+  default:
+    return NmbNormal;
   }
-  return NORMALBASE;
+  return _setstr(&tokens->intpart, p)? NmbSpecial : NmbBufferOverflow;
 }
 
 /* create an ASCIIZ sequence of the integer part,
    set sign and base. */
-static char
+static Error
 int2str(
   p_otokens tokens,
   p_number_desc n,
   char complement)
 {
-  signed char result;
-
   tokens->sign = n->prefix.sign;
   tokens->base = n->prefix.base;
-  result = _pseudo2str(tokens, n->prefix.base);
-  if (result != NORMALBASE)
-    return result;
+  switch (_special2str(tokens, n->prefix.base))
+  {
+    case NmbSpecial: return Success;
+    case NmbBufferOverflow: return IOBufferOverflow;
+    default: break; /* NmbNormal */
+  }
+  /* no special encodings */
   if (complement)
     return _cmplseq2str(&tokens->intpart, &n->intpart);
   return _seq2str(&tokens->intpart, _intdigits(&n->intpart.seq),
@@ -497,22 +421,22 @@ int2str(
 
 /* do some sanity checks, create descriptors of integer and fraction
    part in tokens, set sign and base */
-static int
+static Error
 str2fixp(
   p_number_desc n,
   p_itokens tokens)
 {
-  int result;
+  Error result;
   int maxdigits;
 
   maxdigits = tokens->maxdigits;
   n->prefix.base = tokens->base;
   n->prefix.sign = tokens->sign;
   if (tokens->sign == IO_SIGN_COMPLEMENT
-      && (!_isempty(tokens->fracpart) || !_isempty(tokens->exp)))
-    return IO_ERROR_SIGN_CONFLICT;
+      && (!_isempty(tokens->fracpart) || tokens->exp))
+    return IOInvalidComplement;
   result = str2int(&n->intpart, tokens->intpart, &n->prefix, maxdigits);
-  if (_ispseudobase(n->prefix.base))
+  if (_isspecial(n->prefix.base))
     return result;
   if (!_isempty(tokens->fracpart))
     _str2seq(&n->fracpart, tokens->fracpart,
@@ -520,123 +444,121 @@ str2fixp(
              n->prefix.base, 0);
   if (n->prefix.sign != IO_SIGN_COMPLEMENT
       && n->intpart.seq.digits + n->fracpart.seq.digits == 0)
-    return IO_ERROR_NO_DIGIT;
+    return IONoSignificand;
   if (n->prefix.sign != IO_SIGN_COMPLEMENT
       && _iszero(&n->intpart.seq) && _iszero(&n->fracpart.seq))
     n->prefix.base = IO_BASE_ZERO;
-  return IO_NO_ERROR;
+  return Success;
 }
 
 /* convert integer and fraction part into ASCIIZ sequences */
-static char
+static Error
 fixp2str(
   p_otokens tokens,
   p_number_desc n,
   int scale)
 {
-  char result;
+  Error result;
 
   result = int2str(tokens, n, n->prefix.sign == IO_SIGN_COMPLEMENT);
-  if (result && !_ispseudobase(n->prefix.base))
-    result = _seq2str(&tokens->fracpart, scale, &n->fracpart);
-  return result;
+  if (result != Success || _isspecial(n->prefix.base))
+    return result;
+  return _seq2str(&tokens->fracpart, scale, &n->fracpart);
 }
 
 /* create a descriptor from the digit sequence of the exponent */
-static int
+static Error
 _exp2desc(
   p_number_desc n,
   p_itokens tokens)
 {
-  t_prefix prefix;
-  t_ext_seq_desc digits;
-  int result;
-
-  result = IO_NO_ERROR;
-  if (tokens->expsign != IO_SIGN_NONE || tokens->expbase != IO_BASE_NAN
-      || !_isempty(tokens->exp))
+  if (tokens->expsign != IO_SIGN_NONE || tokens->exp)
   {
-    prefix.sign = tokens->expsign;
-    if (prefix.sign == IO_SIGN_COMPLEMENT)
-      return IO_ERROR_EXP;
-    if (prefix.sign == IO_SIGN_NONE)
-      prefix.sign = IO_SIGN_PLUS;
-    prefix.base = tokens->expbase;
-    _clearint(&digits);
-    result = str2int(&digits, tokens->exp,
-                     &prefix, BITS_IN_EXP);
-    if (result == IO_NO_ERROR)
+    unsigned upperLimit;
+    signed char sign;
+    switch (tokens->base)
     {
-      n->expbase = prefix.base;
-      if (prefix.base != IO_BASE_ZERO
-               && !_seq2value(&n->exp, &digits, prefix.sign))
-        result = IO_OVERFLOW;
+      case 2 : upperLimit = BITS_IN_BINEXP - 1; break;
+      case 8 : upperLimit = BITS_IN_OCTEXP - 1; break;
+      case 16: upperLimit = BITS_IN_HEXEXP - 1; break;
+      default: upperLimit = BITS_IN_EXP - 1; break;
     }
+    upperLimit = (1 << (upperLimit)) - 1;
+    sign = tokens->expsign;
+    switch (sign)
+    {
+      case IO_SIGN_COMPLEMENT:
+        return IOBadExp;
+      case IO_SIGN_NONE:
+        sign = IO_SIGN_PLUS; break;
+      case IO_SIGN_MINUS:
+        ++upperLimit; break;
+      default:;
+    }
+    if (tokens->exp > upperLimit)
+      return IOExpOverflow;
+    n->exp = sign < 0? -(int)(tokens->exp) : tokens->exp;
   }
-  return result;
-}
-
-/* create an ASCIIZ sequence from the exponent value */
-static char
-_desc2exp(
-  p_otokens tokens,
-  p_number_desc n)
-{
-  t_otokens tmptokens;
-  t_number_desc tmpn;
-  char result;
-
-  _cleartokens(&tmptokens);
-  tmptokens.intpart = tokens->exp;
-  _clearnumber(&tmpn);
-  if (n->exp == 0)
-  {
-    tmpn.prefix.base = IO_BASE_ZERO;
-    tmpn.prefix.sign = IO_SIGN_NONE;
-  }
-  else
-  {
-    tmpn.prefix.base = n->expbase;
-    tmpn.prefix.sign = n->exp < 0? IO_SIGN_MINUS : IO_SIGN_PLUS;
-  }
-  _value2seq(&tmpn.intpart, &n->exp, n->expbase);
-  result = int2str(&tmptokens, &tmpn, 0);
-  tokens->expsign = tmptokens.sign;
-  tokens->expbase = tmptokens.base;
-  return result;
+  return Success;
 }
 
 /* create a descriptor from the floating point number given in
    tokens */
-int
+Error
 str2desc(
   p_number_desc n,
   p_itokens tokens)
 {
-  int result;
+  Error result;
 
   _clearnumber(n);
   result = str2fixp(n, tokens);
-  if (result == IO_NO_ERROR)
+  if (result == Success)
     result = _exp2desc(n, tokens);
-  if (result != IO_NO_ERROR)
+  if (result != Success)
     n->prefix.base = IO_BASE_NAN;
   return result;
 }
 
-char
+Error
 desc2str(
   p_otokens tokens,
   p_number_desc n,
   int scale)
 {
-  char result;
+  Error result;
 
   result = fixp2str(tokens, n, scale);
-  if (result && !_ispseudobase(n->prefix.base)
-      && n->expbase != IO_BASE_NAN)
-    return _desc2exp(tokens, n);
-  return result;
+  if (result != Success || _isspecial(n->prefix.base))
+    return result;
+  tokens->exp = n->exp;
+  return Success;
+}
+
+Error
+exp2str(
+  p_buffer dest,
+  int exp,
+  char base)
+{
+  char tmp[BITS_IN_EXP + 3];
+  int idx = 0;
+  int di = 0;
+  if (exp < 0)
+    exp = -exp;
+  while (exp != 0)
+  {
+    tmp[idx++] = hexdigits[exp % base];
+    exp /= base;
+  }
+  if (idx == 0)
+    tmp[idx++] = hexdigits[0];
+  if (dest->sz <= idx)
+    return IOBufferOverflow;
+  for (; --idx >= 0;)
+    dest->buf[di++] = tmp[idx];
+  dest->buf[di] = 0;
+  return Success;
 }
 
 
@@ -645,9 +567,9 @@ desc2str(
 static t_ioparams stdioparams[4] =
 {
   {10, 10, '.', "0d", "eE(", "  )", "", DECPRECISION},
-  {16, 16, '.', "0x", "(", ")", "sF", HEXPRECISION},
-  {2, 16, '.', "0b", "(", ")", "s1", BINPRECISION},
-  {8, 8, '.', "0o", "(", ")", "s7", OCTPRECISION}
+  {16, 10, '.', "0x", "(", ")", "sF", HEXPRECISION},
+  {2, 10, '.', "0b", "(", ")", "s1", BINPRECISION},
+  {8, 10, '.', "0o", "(", ")", "s7", OCTPRECISION}
 };
 
 enum {idzero, idx10, idx16, idx2, idx8, idxcount};
@@ -774,6 +696,11 @@ float_stdconvert()
   setdefaultbase(10);
 }
 
+const char* basePrefix(char base)
+{
+  return getioparams(base)->basetag;
+}
+
 static signed char
 _parsesign(
   const char** buf)
@@ -835,7 +762,7 @@ _parsecmpl(
   return lg > 0;
 }
 
-int
+Error
 parse(
   p_itokens tokens,
   const char** buffer)
@@ -850,8 +777,7 @@ parse(
   char* expend;
 
   tokens->fracpart = NULL;
-  tokens->exp = NULL;
-  tokens->expbase = IO_BASE_NAN;
+  tokens->exp = 0;
   tokens->expsign = IO_SIGN_NONE;
   tokens->maxdigits = 0;
   dot = '.';
@@ -878,7 +804,7 @@ parse(
   if (_parsecmpl(&p, tokens->base))
   {
     if (tokens->sign != IO_SIGN_NONE)
-      return IO_ERROR_SIGN_CONFLICT;
+      return IOInvalidComplement;
     tokens->sign = IO_SIGN_COMPLEMENT;
   }
   tokens->intpart = _scandigits(&p, base);
@@ -889,20 +815,31 @@ parse(
   }
   if (!tokens->intpart && !tokens->fracpart
       && tokens->sign != IO_SIGN_COMPLEMENT)
-    return IO_ERROR_NO_DIGIT;
+    return IONoSignificand;
   expchar = strchr(expbegin, *p);
   if (!_isempty(expchar))
   {
+    const char* expptr;
+    int i;
+    int e = 0;
+    int expbase;
     ++p;
     idx = expchar - expbegin;
     tokens->expsign = _parsesign(&p);
-    tokens->expbase = _parsebase(&p, base);
-    tokens->exp = _scandigits(&p, tokens->expbase);
-    if (!tokens->exp || (*(expend + idx) != ' ' && *(expend + idx) != *p))
-      return IO_ERROR_EXP;
+    expbase = _parsebase(&p, base);
+    expptr = _scandigits(&p, expbase);
+    if (!expptr || (*(expend + idx) != ' ' && *(expend + idx) != *p))
+      return IOBadExp;
+    for (i = 0; i < p-expptr; ++i)
+    {
+      if (!_checkmul(&e, expbase)
+           || !_checkadd(&e, _ascii2digit(*(expptr+i))))
+        return IOExpOverflow;
+    }
+    tokens->exp = e;
   }
   *buffer = p;
-  return IO_NO_ERROR;
+  return Success;
 }
 
 static char
@@ -952,7 +889,7 @@ _cattoken(
   char enable)
 {
   if (enable && !_isempty(token))
-   strcat(buf, token);
+    strcat(buf, token);
 }
 
 int
@@ -960,6 +897,7 @@ cattokens(
   char* buf,
   int bufsz,
   p_otokens tokens,
+  signed char expbase,
   unsigned flags)
 {
   int sz;
@@ -983,10 +921,13 @@ cattokens(
   char printexpbase;
   char printexpbegin;
   char printexpend;
+  char exp[BITS_IN_BINEXP+2];
+  t_buffer expBuf;
 
+  expBuf.sz = sizeof(exp);
+  expBuf.buf = exp;
   cbuf[1] = '\0';
   fraclg = 0;
-  ioparams = getioparams(IO_BASE_DEFAULT);
   if (!_isempty(tokens->fracpart.buf))
   {
     fraclg = strlen(tokens->fracpart.buf) - 1;
@@ -995,19 +936,19 @@ cattokens(
         --fraclg;
     ++fraclg;
   }
+  ioparams = getioparams(IO_BASE_DEFAULT);
   base = tokens->base;
+  printbasetag = !_isspecial(base)
+                 && (flags & IO_FLAG_SUPPRESS_BASETAG) == 0
+                 && (ioparams == NULL || ioparams->base != base);
+  ioparams = getioparams(base);
   basetag = _decodebase(base);
   cmpltag = _decodecomplement(tokens->sign, base);
-  expbasetag = _decodebase(tokens->expbase);
-  printbasetag = !_ispseudobase(base)
-                  && (flags & IO_FLAG_SUPPRESS_BASETAG) == 0
-                  && (ioparams == NULL
-                      || ioparams->base != base);
+  expbasetag = NULL;
   if (base == IO_BASE_DEFAULT)
-    flags |= IO_FLAG_SUPPRESS_DOT + IO_FLAG_SUPPRESS_LDG_ZERO;
+    flags |= IO_FLAG_SUPPRESS_DOT | IO_FLAG_SUPPRESS_LDG_ZERO;
   if ((flags & IO_FLAG_SHOW_BASE) != 0)
     printbasetag = 1;
-  ioparams = getioparams(base);
   printcmpl = tokens->sign == IO_SIGN_COMPLEMENT
                   && (flags & IO_FLAG_SUPPRESS_CMPL) == 0;
   printsign = !printcmpl
@@ -1017,18 +958,23 @@ cattokens(
   printleading0 = _isempty(tokens->intpart.buf)
                   && (flags & IO_FLAG_SUPPRESS_LDG_ZERO) == 0;
   printdot = fraclg > 0 || (flags & IO_FLAG_SUPPRESS_DOT) == 0;
-  printexp = !_isempty(tokens->exp.buf)
+  printexp = base != IO_BASE_NAN && base != IO_BASE_ZERO
              && ((flags & IO_FLAG_SUPPRESS_EXPZERO) == 0
-                 || strcmp(tokens->exp.buf, "0") != 0);
-  printexpsign = tokens->expsign != IO_SIGN_NONE
-                 && (tokens->expsign != IO_SIGN_PLUS
-                     || (flags & IO_FLAG_SUPPRESS_EXPPLUS) == 0);
-  printexpbase = expbasetag != NULL
-                 && (flags & IO_FLAG_SUPPRESS_EXPBASE) == 0
-                 && (_isempty(basetag) 
-                     || strcmp(basetag, expbasetag) != 0);
-  if ((flags & IO_FLAG_SHOW_EXPBASE) != 0)
-    printexpbase = 1;
+                  || tokens->exp != 0);
+  if (printexp)
+  {
+    if (expbase < 2)
+      expbase = ioparams->expbase;
+    expbasetag = _decodebase(expbase);
+    printexpsign = tokens->exp < 0
+                   || (flags & IO_FLAG_SUPPRESS_EXPPLUS) == 0;
+    printexpbase = expbasetag != NULL
+                   && (flags & IO_FLAG_SUPPRESS_EXPBASE) == 0
+                   && (_isempty(basetag)
+                       || strcmp(basetag, expbasetag) != 0);
+    if ((flags & IO_FLAG_SHOW_EXPBASE) != 0)
+      printexpbase = 1;
+  }
   dot = '.';
   expbegin = "(";
   expend = ")";
@@ -1056,13 +1002,14 @@ cattokens(
   sz += fraclg;
   if (printexp)
   {
+    exp2str(&expBuf, tokens->exp, expbase);
     if (printexpbegin)
       ++sz;
     if (printexpsign)
       sz += 1;
     if (printexpbase)
       sz += strlen(expbasetag);
-    sz += strlen(tokens->exp.buf);
+    sz += strlen(expBuf.buf);
     if (printexpend)
       ++sz;
   }
@@ -1083,10 +1030,10 @@ cattokens(
     {
       cbuf[0] = *expbegin;
       _cattoken(buf, cbuf, printexpbegin);
-      cbuf[0] = _decodesign(tokens->expsign);
+      cbuf[0] = _decodesign(tokens->exp < 0? -1:1);
       _cattoken(buf, cbuf, printexpsign);
       _cattoken(buf, expbasetag, printexpbase);
-      strcat(buf, tokens->exp.buf);
+      strcat(buf, expBuf.buf);
       cbuf[0] = *expend;
       _cattoken(buf, cbuf, printexpend);
     }
